#include "wiced_platform.h"
#include "DB1502_Bosch.h"

static wiced_i2c_device_t  _i2c_dev;
uint8_t BME680_hang;
uint8_t BME680_rxLast[10];

uint8_t BMA280_hang;
uint8_t BMA280_rxLast[7];
uint32_t reset_cnt=0;


void  DB1502_Sensor_Init(void)
{
	wiced_result_t result;
    _i2c_dev.port           = WICED_I2C_1;
    _i2c_dev.address        = BOSCH_BME680_DEVICE_ADDR;
    _i2c_dev.address_width  = I2C_ADDRESS_WIDTH_7BIT;
    _i2c_dev.flags          = I2C_DEVICE_NO_DMA;
    _i2c_dev.speed_mode     = I2C_STANDARD_SPEED_MODE;
    result=wiced_i2c_init(&_i2c_dev);

    if(result==WICED_SUCCESS)
    	WPRINT_APP_INFO(("DB1502_Sensor_Init OK.\r\n"));
    else
    	WPRINT_APP_INFO(("DB1502_Sensor_Init Fail.\r\n"));

    BME680_hang=0;
    memset(BME680_rxLast, 0, sizeof(BME680_rxLast));
    
    BMA280_hang=0;
    memset(BMA280_rxLast, 0, sizeof(BMA280_rxLast));
}	

void DB1502_Sensor_UnInit(void)
{
	wiced_i2c_deinit(&_i2c_dev);
}

uint16_t  BME680_GetVendorID(void)
{	 	
    return 0; 	
}	

/* BME680 register format
+-----------------------------------------------+
| 0x09	|  IAQ accuracy						   	|
+-----------------------------------------------+
| 0x08	|  IAQ LSB								|
+-----------------------------------------------+
| 0x07	|  IAQ MSB								|
+-----------------------------------------------+
| 0x06	|  Humidity LSB							|
+-----------------------------------------------+
| 0x05	|  Humidity MSB							|
+-----------------------------------------------+
| 0x04	|  Pressure LSB							|
+-----------------------------------------------+
| 0x03	|  Pressure MSB							|
+-----------------------------------------------+
| 0x02	|  Bit[5]   : sign  					|
|       |  bit[4:0] :Temperature LSB (decimal)	|
+-----------------------------------------------|
| 0x01	|  Temperature MSB (integer)			|
+-----------------------------------------------+
| 0x00	|  Chip ID								|
+-----------------------------------------------+
Chip ID = 0xAA
Temperature (-40.0 ~ +85.0) = sign* (Temperature MSB + 0.1*Temperature LSB)
sign bit = 0x0b, sign = +1
sign bit = 0x01b, sign = -1
Pressure (300.0~1200.0) = Pressure MSB*10+ Pressure LSB*0.1
Humidity (0.0 ~ 100.0) = Humidity MSB+ Humidity LSB*0.1
IAQ (0~500 decimal) = IAQ MSB*10 + IAQ LSB
IAQ accuracy = 0~3
*/
wiced_bool_t BME680_GetSensorData(float *Temperature, float *Pressure, float *Humidity, uint16_t *IAQ, uint8_t *Accuracy, uint8_t *raw)
{
	int32_t    i;
	float   _temp;
    float   _prss;
	float   _hmdty;
    uint16_t _iaq;

    wiced_i2c_message_t msg;
	uint8_t reg=BOSCH_BME680_DEVICE_ID;
	uint8_t rxBuff[10];
	wiced_result_t result;

	memset(rxBuff, 0, sizeof(rxBuff));

	wiced_i2c_init_tx_message(&msg, &reg, sizeof(reg), 1, I2C_DEVICE_NO_DMA);

    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);

    if(result != WICED_SUCCESS){      
    	WPRINT_APP_INFO(("BME680_err1\r\n"));
       	return WICED_FALSE;
    }
    wiced_rtos_delay_milliseconds(10);
    
    wiced_i2c_init_rx_message(&msg, rxBuff, sizeof(rxBuff), 1, I2C_DEVICE_NO_DMA);
    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);

    if(result==WICED_SUCCESS)
	{
    	WPRINT_APP_INFO(("BME680_GetSensorData OK.\r\n"));

	    for(i=0; i<sizeof(rxBuff); i++)
            raw[i]=rxBuff[i];
        
		if(rxBuff[2] & 0x20){
			_temp=(-1.0)*(rxBuff[1] + (0.1*(rxBuff[2]& 0x1f)));
		}
        else{
            _temp=rxBuff[1] + (0.1*(rxBuff[2]& 0x1f));
        }
        _prss = (rxBuff[3]*10) + (rxBuff[4]*0.1);
		_hmdty = rxBuff[5] + (rxBuff[6]*0.1);
        _iaq = (rxBuff[7]*10) + rxBuff[8];
	}
    else{
    	WPRINT_APP_INFO(("BME680_GetSensorData Fail.\r\n"));
    	return WICED_FALSE;
    }
	if(Temperature!=NULL)
		*Temperature = _temp;
    if(Pressure!=NULL)
		*Pressure = _prss;
	if(Humidity!=NULL)
		*Humidity = _hmdty;
	if(IAQ!=NULL)
		*IAQ = _iaq;
    if(Accuracy!=NULL)
        *Accuracy = rxBuff[9];

    if(memcmp(BME680_rxLast, rxBuff, sizeof(BME680_rxLast))==0){
        BME680_hang=1;
    }
    else{
        memcpy(BME680_rxLast, rxBuff, sizeof(BME680_rxLast));
        BME680_hang=0;
    }

    return WICED_TRUE;
}

/* BMA280 register format
Acceleration datas format is like following.
Each axis accel data is 14-bit. Therefore there are totally 2^14 = 16384 outputs.
The unit of output is called LSB in our datasheet.
The data has sign, so the output range is [-8192, 8191].
14-bit is stored in 2 bytes, MSB byte and LSB byte.
The highest bit of MSB byte is the sign bit to indicate data is positive or negative

Default range setting is 2G, so resolution of accel data is 8192 LSB/2G = 4096/G.
1G = 4096LSB.
1LSB = (1/4096) G = 0.244mg
*/
wiced_bool_t BMA280_GetSensorData(int *gX, int *gY, int *gZ, uint8_t *raw)
{
	int32_t    i;
    uint16_t   u16_X;
    uint16_t   u16_Y;
    uint16_t   u16_Z;

    wiced_i2c_message_t msg;
	uint8_t reg=(BOSCH_BMA280_ACC_X_MSB);     // meet with BOSCH_BMA280 bug
	uint8_t rxBuff[6];
	wiced_result_t result;    

	memset(rxBuff, 0, sizeof(rxBuff));

	wiced_i2c_init_tx_message(&msg, &reg, sizeof(reg), 1, I2C_DEVICE_NO_DMA);

    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);
    if(result != WICED_SUCCESS){  
    	WPRINT_APP_INFO(("BMA280_err1\r\n"));
       	return WICED_FALSE;
    }
    wiced_rtos_delay_milliseconds(10);
    
    wiced_i2c_init_rx_message(&msg, rxBuff, sizeof(rxBuff), 1, I2C_DEVICE_NO_DMA);
    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);

    if(result==WICED_SUCCESS)
	{
    	WPRINT_APP_INFO(("BMA280_GetSensorData OK.\r\n"));
        for(i=0; i<sizeof(rxBuff); i++)
            raw[i]=rxBuff[i];

        u16_X = ((rxBuff[0]<<8) | (rxBuff[1])) >> 2;
		u16_Y = ((rxBuff[2]<<8) | (rxBuff[3])) >> 2;
        u16_Z = ((rxBuff[4]<<8) | (rxBuff[5])) >> 2;
	}
    else{
    	WPRINT_APP_INFO(("BMA280_GetSensorData Fail.\r\n"));
    	return WICED_FALSE;
    }
	if(gX!=NULL){
        if(u16_X & 0x2000)
            *gX = -1 * ((((~u16_X+1) & 0x1FFF)*1000)/4096);
        else
		    *gX = ((u16_X & 0x1FFF)*1000)/4096;
    }
    if(gY!=NULL){
        if(u16_Y & 0x2000)
            *gY = -1 * ((((~u16_Y+1) & 0x1FFF)*1000)/4096);
        else
		    *gY = ((u16_Y & 0x1FFF)*1000)/4096;
    }
	if(gZ!=NULL){
        if(u16_Z & 0x2000)
            *gZ = -1 * ((((~u16_Z+1) & 0x1FFF)*1000)/4096);
        else
		    *gZ = ((u16_Z & 0x1FFF)*1000)/4096;
    }
    
    if(memcmp(BMA280_rxLast, rxBuff, sizeof(BMA280_rxLast))==0){
        BMA280_hang=1;
    }
    else{
        memcpy(BMA280_rxLast, rxBuff, sizeof(BMA280_rxLast));
        BMA280_hang=0;
    }

    if((BME680_hang==1) && (BMA280_hang==1)){
        WPRINT_APP_INFO(("Reset Bosch MCU(%d).\r\n", ++reset_cnt));
        wiced_gpio_output_low(nCB_RESET_OUT);
        wiced_rtos_delay_milliseconds(100);
        wiced_gpio_output_high(nCB_RESET_OUT);
    }

    return WICED_TRUE;
}

